#python script ran on Dell Studio 1558, Windows 7
#Python 2.7.8 (default, Jun 30 2014, 16:03:49) [MSC v.1500 32 bit (Intel)] on win32
#serial out using TTL-232R-3V3 cable
#for details of interface to HPLC pump
#via Raspberry Pi see (freely avaiable supp info):
#Synlett 2016; 27(01): 164-168
#DOI: 10.1055/s-0035-1560975
#https://www.thieme-connect.com/products/ejournals/abstract/10.1055/s-0035-1560975

import cv2
import numpy as np
import time
import serial

camport = 0 #set accordingly if you have multiple cameras attached. 

cam = cv2.VideoCapture(camport)

ramp_frames = 20

drag_start = None
drag_end = None
box = (0,0,0,0)

interface = 0.5 #reference level set to half way between limits

kernel = np.ones((5,5),np.uint8)
kernel2 = cv2.getStructuringElement(cv2.MORPH_ELLIPSE,(5,5))

huevar = 20 #sets hue tolerance to +- 20 units. Set accordingly. 

rpos = None

ser = serial.Serial()
ser.baudrate = 9600
ser.port = 'COM23' #set this to the correct value for your serial/com port
ser.open()




def poscalc(refpt1, refpt2, scanpt):

    percent = (float(scanpt[1])-float(refpt2[1]))/(float(refpt1[1])-float(refpt2[1]))
    return percent

def posdo(fract):

    
    global ser

    global pumpspeed
    global now
    
    error = interface - fract
    print  'error = ' + str(error)
    
    if fract < 0.0:
        fract = 0.0
    a = (2*fract)
    b = pow(a,3.32)
    c = int(b*10)
    if c > 99:
        c = 99
    print 'b = ' + str(b)
    print 'c = ' + str(c)
    ser.write(str(c).zfill(2) + '\n') 

    print 'str c = %s' % str(c)

def trackcam(cam, hueval):
    global huevar

    hue1 = hueval - huevar
    hue2 = hueval + huevar

    go = True
    while go:
        retval, f = cam.read()
        hsv = cv2.cvtColor(f, cv2.COLOR_BGR2HSV)
        lower = np.array([hue1, 50, 50])
        higher = np.array([hue2, 255, 255])

        mask = cv2.inRange(hsv, lower, higher)

        opened2 = cv2.morphologyEx(mask, cv2.MORPH_OPEN, kernel2)

        _, contours, heirarchy = cv2.findContours(opened2, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)

        areas = [cv2.contourArea(c) for c in contours]

        if len(areas) > 0:
        
            max_index = np.argmax(areas)
            cnt = contours[max_index]

            M = cv2.moments(cnt)

            cx = int(M['m10']/M['m00'])
            cy = int(M['m01']/M['m00'])
            cv2.circle(f, (cx, cy), 10, (255,255,0), 3)
            
        cv2.imshow('mask', mask)

        fract = poscalc(refpos2, refpos1, (cx, cy))
        posdo(fract)

        print fract

        cv2.circle(f, refpos1, 5, (255,100,100), thickness = 2)
        cv2.circle(f, refpos2, 5, (255,100,100), thickness = 2)
        
        cv2.imshow('frame', f)
        
        k = cv2.waitKey(5)
        if k == 27:
            cv2.destroyAllWindows()
            break

def takepic():
    retval, im = cam.read()
    return im

def on_mouse2(event, x, y, flags, param):
    global rpos
    if event == cv2.EVENT_LBUTTONDOWN:
        rpos = (x,y)

def getrefpos(image):
    global rpos
    rpos = None
    im = image
    cv2.namedWindow('select refpos', 1)
    cv2.setMouseCallback('select refpos', on_mouse2)
    a = True
    while a:
        if rpos:
            a = False
            cv2.destroyWindow('select refpos')
            return rpos
        cv2.imshow('select refpos', im)
        k = cv2.waitKey(7)
        if k == 27:
            cv2.destroyWindow('select refpos')
            return None

def on_mouse(event, x, y, flags, param):
    global drag_start
    global drag_end
    global box
    if event == cv2.EVENT_LBUTTONDOWN:
        drag_end = False
        drag_start = (x,y)
        print drag_start
        print 'left down'
    if event == cv2.EVENT_LBUTTONUP:
        drag_end = (x,y)
        print drag_end
        print 'left up'
        
    if drag_start and drag_end:
        
        xmin = min (drag_start[0], drag_end[0])
        ymin = min (drag_start[1], drag_end[1])
        xmax = max (drag_start[0], drag_end[0])
        ymax = max (drag_start[1], drag_end[1])
        box = (xmin, ymin, xmax-xmin, ymax-ymin)
    if box:
        print box

def getbox(image):
    print getbox
    im = image
    cv2.namedWindow('select', 1)
    cv2.setMouseCallback('select', on_mouse)
    a = True
    while a:
        if box[2]>0 and box[3]>0:
            cv2.destroyWindow('select')
            return box
        cv2.imshow('select', im)
        k = cv2.waitKey(7)
        if k == 27:
            cv2.destroyWindow('select')
            return None
        
def gethist(image):
    global box
    im = image
    im2 = im[box[1]:(box[1] + box[3]), box[0]: (box[0]+box[2])]
    hsv = cv2.cvtColor(im2, cv2.COLOR_BGR2HSV)
    hist = cv2.calcHist([hsv], [0], None, [180], [0, 180])
    
    return hist


for i in range(ramp_frames):
    temp = takepic()

capture = takepic()

refpos1 = getrefpos(capture)
print refpos1
refpos2 = getrefpos(capture)
print refpos2

box1 = getbox(capture)
print box1

hist = gethist(capture)

m = np.argmax(hist)
print 'strongest hue is ' + str(m)

trackcam(cam, m)

del(cam)
    
